import React from 'react';
import {
  useAppDispatch,
  useGraphQueryResult,
  useSchemaGraph,
  useQuerybuilderGraph,
  useSearchResult,
  AppDispatch,
} from '@graphpolaris/shared/lib/data-access';
import { filterData } from './similarity';
import {
  addSearchResultData,
  addSearchResultSchema,
  addSearchResultQueryBuilder,
  resetSearchResults,
  CATEGORY_KEYS,
} from '@graphpolaris/shared/lib/data-access/store/searchResultSlice';
import { Close } from '@mui/icons-material';

const SIMILARITY_THRESHOLD = 0.7;

const CATEGORY_ACTIONS: {
  [key in CATEGORY_KEYS]: (payload: { nodes: Record<string, any>[]; edges: Record<string, any>[] }, dispatch: AppDispatch) => void;
} = {
  data: (payload: { nodes: Record<string, any>[]; edges: Record<string, any>[] }, dispatch: AppDispatch) => {
    dispatch(addSearchResultData(payload));
  },
  schema: (payload: { nodes: Record<string, any>[]; edges: Record<string, any>[] }, dispatch: AppDispatch) => {
    dispatch(addSearchResultSchema(payload));
  },
  querybuilder: (payload: { nodes: Record<string, any>[]; edges: Record<string, any>[] }, dispatch: AppDispatch) => {
    dispatch(addSearchResultQueryBuilder(payload));
  },
};

const SEARCH_CATEGORIES: CATEGORY_KEYS[] = Object.keys(CATEGORY_ACTIONS) as CATEGORY_KEYS[];

export function SearchBar({}) {
  const inputRef = React.useRef<HTMLInputElement>(null);
  const dispatch = useAppDispatch();
  const results = useSearchResult();
  const schema = useSchemaGraph();
  const graphData = useGraphQueryResult();
  const querybuilderData = useQuerybuilderGraph();
  const [search, setSearch] = React.useState<string>('');
  const [inputActive, setInputActive] = React.useState<boolean>(false);

  const dataSources: {
    [key: string]: { nodes: Record<string, any>[]; edges: Record<string, any>[] };
  } = {
    data: graphData,
    schema: schema,
    querybuilder: querybuilderData,
  };

  React.useEffect(() => {
    handleSearch();
  }, [search]);

  const handleSearch = () => {
    let query = search.toLowerCase();
    const categories = search.match(/@[^ ]+/g);

    if (categories) {
      categories.map((category) => {
        query = query.replace(category, '').trim();
        const cat = category.substring(1);

        if (cat in CATEGORY_ACTIONS) {
          const categoryAction = CATEGORY_ACTIONS[cat as CATEGORY_KEYS];
          const data = dataSources[cat];

          const payload = {
            nodes: filterData(query, data.nodes, SIMILARITY_THRESHOLD),
            edges: filterData(query, data.edges, SIMILARITY_THRESHOLD),
          };

          categoryAction(payload, dispatch);
        }
      });
    } else {
      for (const category of SEARCH_CATEGORIES) {
        const categoryAction = CATEGORY_ACTIONS[category];
        const data = dataSources[category];

        const payload = {
          nodes: filterData(query, data.nodes, SIMILARITY_THRESHOLD),
          edges: filterData(query, data.edges, SIMILARITY_THRESHOLD),
        };

        categoryAction(payload, dispatch);
      }
    }
  };

  React.useEffect(() => {
    const handleClickOutside = ({ target }: MouseEvent) => {
      if (inputRef.current && target && !inputRef.current.contains(target as Node)) {
        setSearch('');
        // dispatch(resetSearchResults());
        inputRef.current.blur();
      }
    };
    document.addEventListener('click', handleClickOutside);
    return () => {
      document.removeEventListener('click', handleClickOutside);
    };
  }, []);

  return (
    <div className={`searchbar relative form-control mr-4 ${inputActive || search !== '' ? 'w-96' : 'w-56'} transition-width duration-300`}>
      <div className="relative rounded-md shadow-sm w-full">
        <input
          ref={inputRef}
          value={search}
          onChange={(e) => setSearch(e.target.value)}
          type="text"
          onFocus={() => setInputActive(true)}
          onBlur={() => setInputActive(false)}
          placeholder="Search…"
          className="block w-full rounded-md border-0 py-1.5 pl-2 pr-8 text-gray-900 ring-1 ring-inset ring-gray-300 placeholder:text-gray-400 sm:text-sm sm:leading-6"
        />
        {search !== '' && (
          <div
            className="absolute inset-y-0 right-0 flex items-center cursor-pointer"
            onClick={() => {
              dispatch(resetSearchResults());
              setSearch('');
            }}
          >
            <div className="py-0 px-2">
              <span className="text-gray-400 text-xs">
                <Close />
              </span>
            </div>
          </div>
        )}
      </div>
      {search !== '' && (
        <div className="absolute top-[calc(100%+0.5rem)] z-10 bg-white rounded-none card-bordered w-full overflow-auto max-h-[60vh]">
          {SEARCH_CATEGORIES.map((category, index) => {
            if (results[category].nodes.length > 0 || results[category].edges.length > 0) {
              return (
                <div key={index}>
                  <div className="flex justify-between p-2 text-lg">
                    <p className="text-entity-500 font-bold">{category.charAt(0).toUpperCase() + category.slice(1)}</p>
                    <p className="font-light text-slate-800">{results[category].nodes.length + results[category].edges.length} results</p>
                  </div>
                  <div className="h-[1px] w-full bg-offwhite-300"></div>
                  {Object.values(Object.values(results[category]))
                    .flat()
                    .map((item, index) => (
                      <div
                        key={index}
                        className="flex flex-col hover:bg-slate-300 p-2 cursor-pointer"
                        title={JSON.stringify(item)}
                        onClick={() => {
                          CATEGORY_ACTIONS[category](
                            {
                              nodes: results[category].nodes.includes(item) ? [item] : [],
                              edges: results[category].edges.includes(item) ? [item] : [],
                            },
                            dispatch,
                          );
                        }}
                      >
                        <div className="font-semibold text-md">
                          {item?.key?.slice(0, 18) || item?.id?.slice(0, 18) || Object.values(item)?.[0]?.slice(0, 18)}
                        </div>
                        <div className="font-light text-slate-800 text-xs">{JSON.stringify(item).substring(0, 40)}...</div>
                      </div>
                    ))}
                </div>
              );
            } else return <></>;
          })}
        </div>
      )}
    </div>
  );
}
